"""
Health monitor for agent services.

This module periodically checks the defined health ports of all services.
If a service is not responding on its port, it will be restarted.
"""

import asyncio
from zscams.agent.src.support.logger import get_logger
from zscams.agent.src.core.services import start_service
from zscams.agent.src.core import running_services
from zscams.agent.src.support.network import is_port_open

logger = get_logger("health_monitor")


async def monitor_services(services: list, config_dir: str, interval: int = 60):
    """
    Periodically check service health by port.

    If a service's health port is closed, attempt to restart it.
    """
    logger.info(f"Health monitor started (interval={interval}s)")
    await asyncio.sleep(30)  # Initial delay to allow services to start
    while True:
        for svc in services:
            name = svc["name"]
            health = svc.get("health", {})

            host = health.get("host", "localhost")
            port = health.get("port")
            if not port:
                logger.debug(f"Skipping {name}: no health port defined.")
                continue

            if name in running_services and is_port_open(host, port, logger):
                logger.debug(f"{name} is healthy on {host}:{port}")
                continue

            logger.warning(f"{name} not healthy, restarting...")
            if name in running_services:
                running_services.remove(name)
            try:
                await start_service(svc, config_dir)
                logger.info(f"Restarted service: {name}")
            except Exception as e:
                logger.error(f"Failed to restart {name}: {e}")

        await asyncio.sleep(interval)



import os
import asyncio
from pathlib import Path
import sys
import argparse
from zscams.agent.src.support.configuration import RemoteConfig, config, CONFIG_PATH
from zscams.agent.src.support.filesystem import is_file_exists, resolve_path
from zscams.agent.src.core.tunnel.tls import create_ssl_context
from zscams.agent.src.core.tunnels import start_all_tunnels
from zscams.agent.src.core.services import start_all_services
from zscams.agent.src.core.service_health_check import monitor_services
from zscams.agent.src.core.backend.client import backend_client
from zscams.agent.src.support.logger import get_logger


logger = get_logger("tls_tunnel_main")


def init_parser():
    parser = argparse.ArgumentParser(description="ZSCAMs Agent")
    parser.add_argument(
        "--bootstrap",
        action="store_true",
        help="Run bootstrap process and exit",
    )
    parser.add_argument(
        "--rebootstrap",
        action="store_true",
        help="Run unbootstrap and then the bootstrap processes and exit",
    )
    parser.add_argument(
        "--unbootstrap",
        action="store_true",
        help="Run unbootstrap process and exit",
    )
    parser.add_argument(
        "--update-machine-info",
        action="store_true",
        help="Reinitialize machine info",
    )
    return parser


def ensure_bootstrapped():
    """Ensure the agent is bootstrapped with the backend."""

    custom_config_path = Path(config.get("config_dir", must_resolve=True))
    files_to_ensure = [
        custom_config_path.joinpath(config.get("remote.ca_cert", must_resolve=True)),
        custom_config_path.joinpath(
            config.get("remote.client_cert", must_resolve=True)
        ),
        custom_config_path.joinpath(config.get("remote.client_key", must_resolve=True)),
    ]

    missing_files = []

    for file in files_to_ensure:
        if not is_file_exists(file, logger):
            missing_files.append(file)

    bootstrapped = backend_client.is_bootstrapped()

    if not bootstrapped or missing_files:
        logger.debug("Agent has entry on server: %s", bootstrapped)
        logger.debug(
            "Missing files: %s",
            ", ".join(missing_files) if missing_files else "No missing files",
        )
        logger.error(
            "Agent is not bootstrapped. Please run `zscams --bootstrap` to start the bootstrap process.",
        )
        sys.exit(1)


async def run():
    """Asynchronous main function to start tunnels and services."""

    custom_config_dir = Path(config.get("config_dir", must_resolve=True))

    ssl_context = create_ssl_context(
        config.get("remote", valtyp=RemoteConfig), custom_config_dir
    )
    remote_host = config.get("remote.host")
    remote_port = config.get("remote.port")

    # Start tunnels and wait for readiness
    tunnel_tasks = await start_all_tunnels(
        config.get("forwards", []), remote_host, remote_port, ssl_context
    )

    # Start services that depend on tunnels
    config_dir = os.path.dirname(CONFIG_PATH)
    service_tasks = asyncio.create_task(
        start_all_services(config.get("services", []), config_dir=config_dir)
    )

    # Start health checks
    monitor_task = asyncio.create_task(
        monitor_services(
            config.get("services", []),
            config_dir=config_dir,
            interval=config.get("general", {}).get("health_check_interval", 300),
        )
    )

    logger.info("[*] All tunnels and services started. Press Ctrl+C to stop.")
    await asyncio.gather(*tunnel_tasks, service_tasks, monitor_task)



import platform


def is_linux():
    """Check if the platform is Linux."""
    return platform.system().lower().startswith("linux")


def is_freebsd():
    """Check if the platform is ZscalerOS or FreeBSD"""
    _platform = platform.system().lower()
    return _platform in ("freebsd", "zscaleros")



import os
import shutil
from pathlib import Path
import subprocess
from typing import Optional, Union
from zscams.agent.src.support.logger import get_logger
from zscams.agent.src.support.platform import is_freebsd, is_linux

from .filesystem import write_to_file

logger = get_logger("os_support")


def set_directory_ownership_and_permissions(
    directory_path: Union[str, Path],
    owner_user: Optional[str] = None,
    mod: Optional[int] = None,
):
    if not is_linux() and not is_freebsd():
        logger.error("Unsupported OS to change directory owner or mod.")
        return

    if not owner_user and not mod:
        raise ValueError("You have to provide a user or mod")

    for root, _dirs, files in os.walk(directory_path):
        if owner_user:
            shutil.chown(root, owner_user)

        if mod:
            os.chmod(root, mod)

        for name in files:
            file_path = os.path.join(root, name)
            if owner_user:
                shutil.chown(file_path, owner_user)
            if mod:
                os.chmod(file_path, mod)

    if owner_user:
        shutil.chown(directory_path, owner_user)
    if mod:
        os.chmod(directory_path, mod)


def system_user_exists(username: str):
    try:
        subprocess.run(
            ["id", username],
            check=True,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
        )
        logger.debug("found the system user %s", username)
        return True
    except subprocess.CalledProcessError:
        return False


def create_system_user(username: str):
    # Support both Linux and FreeBSD
    if not (is_linux() or is_freebsd()):
        logger.error(
            "Error creating system user: This script is intended to run on Linux or FreeBSD systems."
        )
        return

    if not username:
        logger.error("Error creating system user: Username must be provided.")
        return

    # Assuming system_user_exists is already updated to handle both
    if system_user_exists(username):
        logger.warning("User '%s' already exists.", username)
        return

    try:
        if is_freebsd():
            cmd = ["sudo", "pw", "useradd", "-n", username, "-m", "-s", "/bin/sh"]
        else:
            # Standard Linux useradd
            cmd = ["sudo", "useradd", "-m", "-s", "/bin/bash", username]

        subprocess.run(cmd, check=True)
        logger.info("System user '%s' created successfully.", username)

    except subprocess.CalledProcessError as e:
        logger.error("Failed to create user '%s': %s", username, e)


def install_service(service_name: str, content: str):
    """
    Main entry point to install services.
    Redirects to systemd for Linux and rc.d for FreeBSD.
    """
    if is_linux():
        install_systemd_service(service_name, content)
    elif is_freebsd():
        install_rc_service(service_name, content)
    else:
        logger.error("Unsupported OS for service installation.")


def install_rc_service(service_name: str, content: str):
    """
    Installs a FreeBSD rc.d script.
    Note: 'content' for FreeBSD should be a valid rc.subr shell script.
    """
    service_path = f"/usr/local/etc/rc.d/{service_name}"

    try:
        # 1. Write the rc script
        logger.debug("Installing FreeBSD rc script: %s", service_path)
        # Using sudo tee to ensure write permissions on restricted paths
        echo_cmd = f"printf '%s' '{content}' | sudo tee {service_path}"
        subprocess.run(echo_cmd, shell=True, check=True, stdout=subprocess.DEVNULL)

        # 2. Make it executable (Crucial for FreeBSD)
        subprocess.run(["sudo", "chmod", "+x", service_path], check=True)

        # 3. Enable and Start
        # In FreeBSD, enabling adds 'service_name_enable="YES"' to /etc/rc.conf
        logger.debug("Enabling and starting FreeBSD service %s...", service_name)
        subprocess.run(["sudo", "sysrc", f"{service_name}_enable=YES"], check=True)
        try:
            subprocess.run(["sudo", "service", service_name, "start"], check=True)
        except subprocess.CalledProcessError:
            logger.warning(
                'Error on starting service "%s" could be already running, trying to restart...',
                service_name,
            )
            subprocess.run(["sudo", "service", service_name, "restart"], check=True)

        logger.info("Service %s installed and started successfully.", service_name)

    except Exception as e:
        logger.error("Failed to install FreeBSD service %s: %s", service_name, e)


def install_systemd_service(service_name: str, content: str):
    service_path = f"/etc/systemd/system/{service_name}"

    try:
        logger.debug("Installing service '%s' content", service_name)
        write_to_file(service_path, content)
        logger.debug(f"Installed {service_name}")
    except PermissionError:
        logger.warning("Permission denied: trying to write with sudo...")
        echo_cmd = f"echo '{content}' | sudo tee {service_path}"
        subprocess.run(echo_cmd, shell=True, check=True, stdout=subprocess.DEVNULL)
        logger.debug("Wrote the service")
    except Exception as e:
        logger.error("Failed to install service %s. %s", service_name, e)
        return

    try:
        logger.debug("Enabling %s...", service_name)
        subprocess.run(["sudo", "systemctl", "enable", service_name], check=True)
        logger.debug("Starting %s...", service_name)
        subprocess.run(["sudo", "systemctl", "start", service_name], check=True)
    except subprocess.CalledProcessError:
        logger.error(
            "Failed to enable/restart zscams service, You might need to do that manually by running\nsudo systemctl enable %s && sudo systemctl start %s",
            service_name,
            service_name,
        )


def remove_service(service_name: str):
    """
    Stops, disables, and removes service configurations for Linux and FreeBSD.
    """
    # Standardize name (remove .service suffix if present for consistency)
    clean_name = service_name.replace(".service", "")

    if not is_linux() and not is_freebsd():
        logger.error("Unsupported OS for service removal.")
        return False

    if is_linux():
        _remove_systemd_service(clean_name)
    elif is_freebsd():
        _remove_rc_service(clean_name)
    return True


def _remove_systemd_service(name: str):
    service_file = f"/etc/systemd/system/{name}.service"
    try:
        logger.info("Stopping and disabling systemd service: %s", name)
        subprocess.run(["sudo", "systemctl", "stop", name], check=False)
        subprocess.run(["sudo", "systemctl", "disable", name], check=False)

        if os.path.exists(service_file):
            subprocess.run(["sudo", "rm", service_file], check=True)
            subprocess.run(["sudo", "systemctl", "daemon-reload"], check=True)
            logger.info("Systemd service %s removed.", name)
    except subprocess.CalledProcessError as e:
        logger.error("Failed to remove systemd service: %s", e)


def _remove_rc_service(name: str):
    rc_file = f"/usr/local/etc/rc.d/{name}"
    try:
        logger.info("Stopping and disabling FreeBSD service: %s", name)
        # 1. Stop the service
        subprocess.run(["sudo", "service", name, "stop"], check=False)

        # 2. Disable in /etc/rc.conf
        # Using sysrc -x removes the variable entirely from rc.conf
        subprocess.run(["sudo", "sysrc", "-x", f"{name}_enable"], check=False)

        # 3. Remove the rc.d script
        if os.path.exists(rc_file):
            subprocess.run(["sudo", "rm", rc_file], check=True)
            logger.info("FreeBSD rc script %s removed.", name)

    except subprocess.CalledProcessError as e:
        logger.error("Failed to remove FreeBSD service: %s", e)


def running_as_root():
    if not is_linux() and not is_freebsd():
        return True
    return os.geteuid() == 0



import requests
import ssl
from urllib3.poolmanager import PoolManager
from requests.adapters import HTTPAdapter


class SNIAdapter(HTTPAdapter):
    def __init__(self, server_hostname, **kwargs):
        self.server_hostname = server_hostname
        super().__init__(**kwargs)

    def init_poolmanager(self, *args, **kwargs):
        context = ssl.create_default_context()
        kwargs["ssl_context"] = context
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        self.poolmanager = PoolManager(*args, **kwargs)
        self.poolmanager.connection_pool_kw["server_hostname"] = self.server_hostname


def create_session_with_sni_adapter(sni: str):
    session = requests.Session()
    adapter = SNIAdapter(sni)
    session.mount("https://", adapter)
    return session



"""
Main entry point for ZSCAMs Agent
"""

import sys
import asyncio
from zscams.agent.src.core.backend.bootstrap import bootstrap
from zscams.agent.src.core.backend.unbootstrap import unbootstrap
from zscams.agent.src.core.backend.update_machine_info import update_machine_info
from zscams.agent.src.support.logger import get_logger
from zscams.agent import init_parser, ensure_bootstrapped, run
from zscams.agent.src.support.os import running_as_root

logger = get_logger("ZSCAMs")


def main():
    """Main function to run the asynchronous main."""
    args = init_parser().parse_args()

    if args.bootstrap:
        try:
            if not running_as_root():
                logger.error("You are NOT running as root.")
                sys.exit(1)
            bootstrap()
            sys.exit(0)
        except Exception as exception:
            logger.error(exception)
            sys.exit(1)

    if args.update_machine_info:
        update_machine_info()
        sys.exit(0)

    if args.unbootstrap:
        try:
            if not running_as_root():
                logger.error("You are NOT running as root.")
                sys.exit(1)
            unbootstrap()
            sys.exit(0)
        except Exception as exception:
            logger.error(exception)
            sys.exit(1)

    if args.rebootstrap:
        try:
            if not running_as_root():
                logger.error("You are NOT running as root.")
                sys.exit(1)
            unbootstrap()
            bootstrap()
            sys.exit(0)
        except Exception as exception:
            logger.error(exception)
            sys.exit(1)

    try:
        ensure_bootstrapped()
        asyncio.run(run())
    except KeyboardInterrupt:
        logger.info("Exiting TLS Tunnel Client")


if __name__ == "__main__":
    main()



"""Dict utilities"""


def merge(source: dict, destination: dict) -> dict:
    """
    Recursively merge two dictionaries. Values from the source dictionary will override those in the destination dictionaries.

    >>> a = { 'first': { 'all_rows': { 'pass': 'dog', 'number': 1 } } }
    >>> b = { 'first': { 'all_rows': { 'fail': 'cat', 'number': 5 } } }
    >>> merge(b, a) == { 'first': { 'all_rows': { 'pass': 'dog', 'fail': 'cat', 'number': 5 } } }
    True
    """
    for key, value in source.items():
        if isinstance(value, dict):
            node = destination.setdefault(key, {})
            merge(value, node)
        else:
            destination[key] = value
    return destination



import subprocess
import sys
import os
import platform
import importlib.util


def is_freebsd():
    _platform = platform.system().lower()
    return _platform == "freebsd" or _platform == "zscaleros"


def is_installed(name):
    return importlib.util.find_spec(name) is not None


def ensure_native_deps():
    if not is_freebsd():
        deps_map = {
            "cryptography": "cryptography",
            "yaml": "pyyaml",
            "psutil": "psutil",
        }
    else:
        py_ver = f"py{sys.version_info.major}{sys.version_info.minor}"
        deps_map = {
            "cryptography": f"{py_ver}-cryptography",
            "yaml": f"{py_ver}-pyyaml",
            "psutil": f"{py_ver}-psutil",
        }

    missing_mods = [mod for mod in deps_map if not is_installed(mod)]
    if not missing_mods:
        return

    if not is_freebsd():
        print(f"--> Installing via pip: {missing_mods}")
        subprocess.run(
            [sys.executable, "-m", "pip", "install"] + missing_mods, check=True
        )
        return

    print(
        f"--> ZscalerOS/FreeBSD detected. Missing requirements: {', '.join(missing_mods)}"
    )

    # Ensure the FreeBSD Mirror is configured
    # We use a custom local file so we don't overwrite Zscaler's system configs
    repo_conf_dir = "/usr/local/etc/pkg/repos"
    repo_conf_file = f"{repo_conf_dir}/FreeBSD.conf"

    if not os.path.exists(repo_conf_file):
        print("--> Mirror not found. Configuring official FreeBSD repository...")
        # Hardcoding the ABI to 13 because ZscalerOS identifies as 42-RELEASE
        mirror_config = (
            "FreeBSD: { "
            'url: "pkg+http://pkg.FreeBSD.org/FreeBSD:13:amd64/latest", '
            'mirror_type: "srv", '
            'signature_type: "fingerprints", '
            'fingerprints: "/usr/share/keys/pkg", '
            "enabled: yes "
            "}"
        )

        try:
            subprocess.run(["sudo", "mkdir", "-p", repo_conf_dir], check=True)
            # Use printf/tee to handle the sudo write to a restricted path
            subprocess.run(
                f"printf '{mirror_config}' | sudo tee {repo_conf_file}",
                shell=True,
                check=True,
                capture_output=True,
            )
            print(
                "--> Mirror configured. Updating package database (this may take a moment)..."
            )
            subprocess.run(["sudo", "pkg", "update", "-f"], check=True)
        except subprocess.CalledProcessError as e:
            print(f"--> Failed to configure mirror. Error: {e}")
            sys.exit(1)

    # Install the missing packages
    targets = [deps_map[m] for m in missing_mods]
    print(f"--> Attempting auto-install of: {', '.join(targets)}")

    try:
        # Install via pkg
        subprocess.run(["sudo", "pkg", "install", "-y"] + targets, check=True)
        print("--> Dependencies installed successfully!")
        print("--> Please restart your command to apply changes.")
        sys.exit(0)
    except subprocess.CalledProcessError:
        print(f"--> Error: Failed to install packages. Check your internet connection.")
        print(f"--> Manual command: sudo pkg install {' '.join(targets)}")
        sys.exit(1)



"""
Configuration loader module
"""

import os
import shutil
from pathlib import Path
from typing import Optional, Type, TypeVar, TypedDict, cast
import yaml

import zscams
from zscams.agent.src.support.dicts import merge
from zscams.agent.src.support.yaml import YamlIndentedListsDumper, resolve_placeholders

ROOT_PATH = Path(zscams.__file__).resolve().parent.joinpath("agent")
CONFIG_PATH = os.path.join(ROOT_PATH.absolute(), "config.yaml")

GetReturnT = TypeVar("GetReturnT")


class MissingConfiguration(BaseException):
    """Error when the requisted key is not found"""


class Configuration:
    """Class to get/set configurations and handle the configurations file."""

    __instance = None
    __config = {}

    def __init__(self):
        self.__load_config()

    def __new__(cls):
        if cls.__instance is None:
            cls.__instance = super().__new__(cls)
        return cls.__instance

    def __load_config(self):
        try:
            with open(CONFIG_PATH, "r", encoding="utf-8") as f:
                self.__config = yaml.safe_load(f)

            user_config = self.__get_user_config()
            if user_config is not None:
                merge(user_config, self.__config)

        except FileNotFoundError:
            shutil.copyfile(ROOT_PATH.joinpath("configuration/config.j2"), CONFIG_PATH)
            self.__load_config()

    def __get_user_config(self):
        if not self.__config or self.__config.get("config_dir") is None:
            return

        user_config_dir = Path(self.__config.get("config_dir", ""))
        if not user_config_dir.exists():
            return

        user_config = user_config_dir.joinpath("config.yaml")
        if not user_config.exists():
            alt_user_config = user_config_dir.joinpath("config.yml")
            if alt_user_config.exists():
                user_config = alt_user_config
            else:
                return

        file_content = user_config.read_text(encoding="utf-8")
        if not file_content.strip():
            return

        return yaml.safe_load(file_content)

    def override_config(self, new_config: dict):
        """
        Override the existing configuration with a new one.
        Args:
            new_config (dict): New configuration dictionary to override the existing one.
        """
        self.__config = new_config

        with open(CONFIG_PATH, "w", encoding="utf-8") as file:
            yaml.dump(
                self.__config,
                file,
                Dumper=YamlIndentedListsDumper,
                default_flow_style=False,
                explicit_start=True,
            )

    def to_dict(self):
        return self.__config

    def get(
        self,
        key: str,
        default: Optional[GetReturnT] = None,
        must_resolve=False,
        valtyp: Type[GetReturnT] = str,
    ) -> GetReturnT:
        """
        Returns the configuration value.

        Args:
            key (str): The configurations dotnotation key to get the value of.
            default (Optional[T]): The default value if the configurations wasn't found.
            must_resolve (bool): Wheither to throw an error if the value wasn't found and if there was no default.
            valtyp (Type[T]): The expected type of the return value.
        """

        keys = key.split(".")
        val = self.__config

        for k in keys:
            if isinstance(val, dict) and k in val:
                val = val.get(k, default)
            elif isinstance(val, list) and int(k) < len(val):
                val = val[int(k)]
            elif default is not None:
                val = default
                break
            elif must_resolve:
                raise MissingConfiguration
            else:
                val = None
                break

        return cast(GetReturnT, val)

    def must_get(
        self,
        key: str,
        default: Optional[GetReturnT] = None,
        valtyp: Type[GetReturnT] = str,
    ) -> GetReturnT:
        return self.get(key, default=default, must_resolve=True, valtyp=valtyp)

    def has(self, key: str):
        try:
            self.get(key, must_resolve=True)
            return True
        except MissingConfiguration:
            return False

    def reinitialize(self, **kwargs):
        """Regenerate the config template with new values"""
        template_path = ROOT_PATH.joinpath("configuration/config.j2")
        with open(template_path, encoding="utf-8") as f:
            template = yaml.safe_load(f)
        resolve_placeholders(template, kwargs)
        self.override_config(template)


class RemoteConfig(TypedDict):
    """Type definition for remote configuration."""

    host: str
    port: str
    verify_cert: bool
    client_key: str
    client_cert: str
    ca_cert: str
    ca_chain: str


config = Configuration()



"""Module for managing machine ID based on MAC address or system-generated ID."""

from pathlib import Path
from typing import Optional

from zscams.agent.src.support.configuration import config
from zscams.agent.src.support.filesystem import ensure_dir


MACHINE_ID_PATH = Path(config.must_get("config_dir")).joinpath("machine_id")


def get_machine_id():
    """Get the machine ID, either from the file or by using the MAC address."""
    try:
        return MACHINE_ID_PATH.read_text("utf-8").strip() or None
    except FileNotFoundError:
        return None


def set_machine_id(machine_id: Optional[str]):
    """Get the machine ID, either from the file or by using the MAC address."""
    ensure_dir(MACHINE_ID_PATH.parent)

    if not machine_id and MACHINE_ID_PATH.exists():
        return MACHINE_ID_PATH.unlink()

    if not machine_id:
        return

    return MACHINE_ID_PATH.write_text(machine_id, "utf-8")



import yaml


class YamlIndentedListsDumper(yaml.Dumper):
    def increase_indent(self, flow=False, indentless=False):
        return super(YamlIndentedListsDumper, self).increase_indent(flow, False)


def resolve_placeholders(obj, values: dict[str, int]):
    """
    Recursively replace '{key}' strings with values[key]
    in dicts, lists, and strings.
    """
    if isinstance(obj, dict):
        for k, v in obj.items():
            obj[k] = resolve_placeholders(v, values)

    elif isinstance(obj, list):
        for i, v in enumerate(obj):
            obj[i] = resolve_placeholders(v, values)

    elif isinstance(obj, str):
        for key, value in values.items():
            placeholder = f"{{{key}}}"
            if obj == placeholder:
                return value
        return obj

    return obj


def assert_no_placeholders_left(obj):
    if isinstance(obj, dict):
        for v in obj.values():
            assert_no_placeholders_left(v)
    elif isinstance(obj, list):
        for v in obj:
            assert_no_placeholders_left(v)
    elif isinstance(obj, str) and obj.startswith("{") and obj.endswith("}"):
        raise ValueError(f"Unresolved placeholder: {obj}")




from pathlib import Path

from zscams.agent.src.support.cli import prompt_auth_info
from zscams.agent.src.support.machine_id import set_machine_id

from .client import backend_client
from zscams.agent.src.support.configuration import config, ROOT_PATH
from zscams.agent.src.support.logger import get_logger
from zscams.agent.src.support.os import remove_service

logger = get_logger("Unbootstrap")


def unbootstrap():
    if not backend_client.is_authenticated():
        username, totp = prompt_auth_info()
        backend_client.login(username, totp)

    backend_client.unbootstrap()
    logger.info(
        "Unbootstrapped from backend and revoked the certificate.",
    )

    remove_local_data()


def remove_local_data():
    custom_config_dir = Path(config.get("config_dir", must_resolve=True))

    cert_files_to_remove = [
        config.get("remote.client_key", must_resolve=True),
        config.get("remote.client_cert", must_resolve=True),
        config.get("remote.ca_chain", must_resolve=True),
    ]

    for cert_file in cert_files_to_remove:
        path = custom_config_dir.joinpath(cert_file)
        if path and path.exists():
            path.unlink()
            logger.debug(f"Removed {cert_file}")

    cache_path = ROOT_PATH.parent.joinpath(
        config.get("bootstrap.cache_dir", must_resolve=True),
        backend_client.MACHINE_INFO_FILENAME,
    )

    if cache_path.exists():
        cache_path.unlink()
        logger.debug("Removed Machine info")

    set_machine_id(None)

    removed_service = remove_service("zscams")
    if removed_service:
        logger.debug("Removed ZSCAMs service")



import asyncio
import os
import sys
import datetime
import psutil
import json
import platform
from zscams.agent.src.support.logger import get_logger
from http.client import HTTPConnection

logger = get_logger("system_monitor")

# Load service-specific params from environment
params_env = os.environ.get("SERVICE_PARAMS")
if not params_env:
    logger.error("SERVICE_PARAMS environment variable not set")
    sys.exit(1)

params = json.loads(params_env)
# -----------------------------
# Configuration
# -----------------------------
SYSLOG_HOST = params.get("remote_host", "localhost")  # remote rsyslog host
SYSLOG_PORT = params.get("remote_port", 514)  # TCP syslog port
EQUIPMENT_NAME = params.get("equipment_name", "connector")
EQUIPMENT_TYPE = params.get("equipment_type", "zpa")
SERVICE_NAME = params.get("service_name", "Zscaler-AppConnector")
HOSTNAME = platform.node()


def network():
    """
    Collect network interfaces and their statistics.
    Returns a dictionary with per-interface
    """
    ifaces = {}
    for iface, stats in psutil.net_io_counters(pernic=True).items():
        ifaces[iface] = {
            "in": {
                "bytes": stats.bytes_recv,
                "packets": stats.packets_recv,
                "errors": getattr(stats, "errin", 0),
                "dropped": getattr(stats, "dropin", 0),
            },
            "out": {
                "bytes": stats.bytes_sent,
                "packets": stats.packets_sent,
                "errors": getattr(stats, "errout", 0),
                "dropped": getattr(stats, "dropout", 0),
            },
        }
    if "lo" in ifaces:
        del ifaces["lo"]
    return ifaces


def memory():
    """
    Get memory and swap usage
    Returns a dictionary with total, used, free, and swap info.
    """
    mem = psutil.virtual_memory()
    swap = psutil.swap_memory()
    mem_remain = mem.total - mem.available
    output_memory = {
        "actual": {
            "free": mem.available,
            "used": {
                "pct": round(mem_remain / mem.total, 4),
                "bytes": mem_remain,
            },
        },
        "total": mem.total,
        "used": {
            "pct": round(mem.percent / 100, 4),
            "bytes": mem.total - mem.available,
        },
        "free": mem.available,
        "swap": {
            "total": swap.total,
            "used": {
                "pct": swap.percent / 100,
                "bytes": swap.used,
            },
            "free": swap.free,
        },
    }
    return output_memory


def cpu():
    """
    Get CPU usage
    Returns a dictionary with total, user, system, idle, and core count.
    """
    times = psutil.cpu_times()
    total = sum(times)
    used = total - times.idle
    cpu_total_pct = round(used / total, 4)
    return {
        "total": {"pct": cpu_total_pct},
        "system": {"pct": round(times.system / total, 4)},
        "user": {"pct": round(times.user / total, 4)},
        "idle": {"pct": round(times.idle / total, 4)},
        "cores": psutil.cpu_count(),
        "iowait": {"pct": round(getattr(times, "iowait", 0) / total, 4)},
        "irq": {"pct": round(getattr(times, "irq", 0) / total, 4)},
        "softirq": {"pct": round(getattr(times, "softirq", 0) / total, 4)},
        "nice": {"pct": round(getattr(times, "nice", 0) / total, 4)},
        "steal": {"pct": round(getattr(times, "steal", 0) / total, 4)},
    }


def load():
    """
    Get CPU load averages over 1, 5, and 15 minutes.
    Returns a dictionary with normalized load per CPU core.
    """
    load1, load5, load15 = psutil.getloadavg()
    cores = psutil.cpu_count()
    cores = float(cores) if cores else 1.0
    return {
        "1": load1,
        "5": load5,
        "15": load15,
        "cores": cores,
        "norm": {
            "1": load1 / cores,
            "5": load5 / cores,
            "15": load15 / cores,
        },
    }


def uptime():
    """
    Return system boot time in seconds since the epoch.
    """
    return {"duration": {"seconds": int(psutil.boot_time())}}


def disk():
    """
    Get disk usage metrics for the root filesystem.
    Returns a dictionary with total, used, and free bytes.
    """
    disk = psutil.disk_usage("/")
    return {"total": disk.total, "used": disk.used, "free": disk.free}


def process(process_names=None):
    """
    Get metrics about running processes by name.
    Default: ["zpa-connector", "zpa-service-edge"]
    Returns a list of dictionaries with process info.
    """
    if process_names is None:
        process_names = ["zpa-connector", "zpa-service-edge"]

    output = []
    for proc in psutil.process_iter(
        [
            "name",
            "pid",
            "ppid",
            "username",
            "memory_info",
            "cpu_times",
            "cmdline",
            "status",
            "create_time",
        ]
    ):
        try:
            if proc.info["name"] in process_names:
                mem = proc.info["memory_info"]
                cpu_pct = proc.cpu_percent(interval=0.1)
                output.append(
                    {
                        "name": proc.info["name"],
                        "pid": proc.info["pid"],
                        "ppid": proc.info["ppid"],
                        "username": proc.info["username"],
                        "memory": {
                            "rss": mem.rss,
                            "vms": mem.vms,
                            "percent": round(proc.memory_percent(), 2),
                        },
                        "cpu": {
                            "percent": cpu_pct,
                            "start_time": datetime.datetime.fromtimestamp(
                                proc.info["create_time"]
                            ).isoformat(),
                        },
                        "cmdline": proc.info["cmdline"],
                        "status": proc.info["status"],
                    }
                )
        except (psutil.NoSuchProcess, psutil.AccessDenied):
            continue
    return output


def collect_all():
    """
    Collect all metrics in a single dictionary.
    """
    return {
        "network": network(),
        "memory": memory(),
        "cpu": cpu(),
        "load": load(),
        "uptime": uptime(),
        "disk": disk(),
        "processes": process(),
    }


def log():
    return {
        "system": collect_all(),
        "beat": {
            "name": EQUIPMENT_NAME,
            "type": EQUIPMENT_TYPE,
            "hostname": HOSTNAME,
        },
        "host": HOSTNAME,
    }


# -----------------------------
# Helper function to log JSON
# -----------------------------
async def send_json_log(payload: dict):
    """
    Send a JSON-formatted log to syslog via TCP
    """
    conn = HTTPConnection(SYSLOG_HOST, SYSLOG_PORT, timeout=10)
    conn.request(
        "POST", "/", json.dumps(payload), headers={"Content-type": "application/json"}
    )
    conn.close()
    logger.info("Metrices collected and sent successfully.")


async def schedule_task(interval):
    while True:
        try:
            await send_json_log(log())
        except Exception as exception:
            print(exception)
        await asyncio.sleep(interval)


if __name__ == "__main__":
    try:
        asyncio.run(schedule_task(30))
    except KeyboardInterrupt:
        logger.info("Exiting system_monitor.py")
        sys.exit(0)



import asyncio
import json
import os
from pathlib import Path
import sys
from zscams.agent.src.support.logger import get_logger
from zscams.agent.src.support.configuration import config

logger = get_logger("autossh_service")

# Load service-specific params from environment
params_env = os.environ.get("SERVICE_PARAMS")
if not params_env:
    logger.error("SERVICE_PARAMS environment variable not set")
    sys.exit(1)

params = json.loads(params_env)

custom_config_dir = Path(config.get("config_dir", must_resolve=True))

LOCAL_PORT = params.get("local_port", 4422)
SERVER_SSH_USER = params.get("server_ssh_user", "ssh_user")
REVERSE_PORT = params.get("reverse_port", 2222)
PRIVATE_KEY = custom_config_dir.joinpath(params.get("private_key"))  # Path to RSA key
SSH_OPTIONS = params.get("ssh_options", [])
CHECK_INTERVAL = 120  #


async def run():
    backoff = 1
    max_backoff = 60
    ssh_cmd = [
        "ssh",
        "-p",
        f"{LOCAL_PORT}",
        "-R",
        f"*:{REVERSE_PORT}:localhost:22",
        f"{SERVER_SSH_USER}@localhost",
        "-N",
    ]

    # Use key file if provided
    if PRIVATE_KEY:
        ssh_cmd += ["-i", str(PRIVATE_KEY)]

    # Add additional SSH options
    ssh_cmd += [str(opt) for opt in SSH_OPTIONS]
    os.chmod(PRIVATE_KEY, 0o700)
    logger.info(f"Starting reverse SSH tunnel: {' '.join(ssh_cmd)}")

    while True:
        try:
            process = await asyncio.create_subprocess_exec(
                *ssh_cmd, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE
            )
            logger.info(f"SSH tunnel started (PID={process.pid})")
            stdout, stderr = await process.communicate()
            if stdout:
                logger.info(stdout.decode())
            if stderr:
                logger.warning(stderr.decode())
            returncode = process.returncode
            logger.warning(f"SSH tunnel exited with code {returncode}")
        except Exception as e:
            logger.error(f"SSH tunnel failed: {e}")

        logger.info(f"Reconnecting in {backoff} seconds...")
        await asyncio.sleep(backoff)
        backoff = min(backoff * 2, max_backoff)


if __name__ == "__main__":
    try:
        asyncio.run(run())
    except KeyboardInterrupt:
        logger.info("Exiting autossh_service.py")
        sys.exit(0)
